package org.processmining.plugins.properties.auxiliary;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.plugins.properties.processmodel.configurable.ConfigurableProperty;

/**
 * 
 * @author DSchunse
 *
 */
public class TransitivePreset {

	public static List<ConfigurableProperty<?>> computeOrderingProperties(Collection<ConfigurableProperty<?>> properties){
		try{
			List<ConfigurableProperty<?>> orderedProperties = new ArrayList<ConfigurableProperty<?>>();
			List<ConfigurableProperty<?>> inputProperties = new ArrayList<ConfigurableProperty<?>>(properties);
			// we assume an ordering is possible, and we employ an incremental approach i.e., for whom the preset is fixed we add to the list
			while(!inputProperties.isEmpty()){
				boolean elementAdded = false;
				for(ConfigurableProperty<?> prop: inputProperties){
					if(satisfiedPrefix(orderedProperties, prop)){
						// add this property
						orderedProperties.add(prop);
						inputProperties.remove(prop);
						elementAdded = true;
						break;
					}
				}
				if(!elementAdded){
					// seems we cannot solve this puzzle
					throw new Exception("We cannot solve the puzzle");
				}
			}
			return orderedProperties;
		} catch (Exception e){
			System.out.print(e);
		}
		return null;
	}
	
	private static boolean satisfiedPrefix(List<ConfigurableProperty<?>> elements, ConfigurableProperty<?> element) throws InstantiationException, IllegalAccessException{
		Collection<Class<? extends Property<?>>> prefix = element.getDependencies();
		for(Class<? extends Property<?>> prop: prefix){
			if(!elements.contains(prop.newInstance())){
				return false;
			}
		}
		return true;
	}
	
}
