package org.processmining.plugins.properties.processmodel.abstractproperty;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.auxiliary.PerformanceOperations;

/**
 * 
 * @author DSchunse
 *
 */
@SuppressWarnings("serial")
public abstract class PointProbabilityOrder extends PropertyListDouble {

	public ArrayList<Double> getDefaultValue() {
		return new ArrayList<Double>();
	}
	
	@SuppressWarnings("unchecked")
	public int compare(Object o1, Object o2) {
		List<Double> o1cast = (List<Double>) o1;
		List<Double> o2cast = (List<Double>) o2;
		// let's make a map out of it to be used in the remainder 
		Map<Double, Double> o1Map = createMap(o1cast);
		Map<Double, Double> o2Map = createMap(o2cast);
		// let's fit a 95% confidence interval
		Pair<Double, Double> o1cast95 = PointProbability.ninetyFivePercentConfidenceInterval(o1cast.size(), o1Map);
		Pair<Double, Double> o2cast95 = PointProbability.ninetyFivePercentConfidenceInterval(o2cast.size(), o2Map);
		
		if((o1cast95.getFirst() <= o2cast95.getFirst() && 
				o2cast95.getFirst() <= o1cast95.getSecond()) ||
				(o2cast95.getFirst() <= o1cast95.getFirst() && 
						o1cast95.getFirst() <= o2cast95.getSecond())){
			return 0;
		}
		else{
			return Double.compare(PerformanceOperations.mean(o1Map), PerformanceOperations.mean(o2Map));
		}
	}
	
	private Map<Double, Double> createMap(List<Double> values){
		double weight = 1d/values.size();
		Map<Double, Double> ret = new HashMap<>();
		for(Double d: values){
			Double prop = ret.get(d);
			if(prop == null){
				prop = 0d;
			}
			prop += weight;
			ret.put(d, prop);
		}
		return ret;
	}
}
