package org.processmining.plugins.properties.processmodel.configurable.abstractproperty;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.auxiliary.HelperFunctions;
import org.processmining.plugins.properties.auxiliary.XMLHelper;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.plugins.properties.processmodel.abstractproperty.PropertyList;
import org.processmining.plugins.properties.processmodel.configurable.ConfigurableProperty;
import org.processmining.plugins.properties.processmodel.configurable.ConfigurablePropertyConfiguration;
import org.processmining.plugins.properties.processmodel.gui.GUIProperty;
import org.processmining.plugins.properties.processmodel.gui.impl.GUIConfigurablePropertyList;

/**
 * 
 * @author DSchunse
 *
 * @param <K>
 */
@SuppressWarnings("serial")
public abstract class ConfigurablePropertyList<K> extends ConfigurableHighLevelProperty<List<K>> {

	public Pair<List<K>, List<K>> getDefaultValue() {
		return new Pair<List<K>, List<K>>(new ArrayList<K>(), new ArrayList<K>());
	}
	
	public GUIProperty<Pair<List<K>, List<K>>> getGUIProperty() throws InstantiationException, IllegalAccessException {
		return new GUIConfigurablePropertyList<K>();
	}
	
	@SuppressWarnings("unchecked")
	public String marshall(Object value) {
		// we have a pair
		Pair<K, K> elements = (Pair<K, K>) value;
		// now we can use the existing unmarshall
		String ret = "<valueEntryElements>\n";
		ret += PropertyList.marshallStatic(elements.getFirst());
		ret += "</valueEntryElements>\n";
		ret += "<valueEntryRemovable>\n";
		ret += PropertyList.marshallStatic(elements.getSecond());
		ret += "</valueEntryRemovable>\n";
		return ret;
	}
	
	protected Pair<Object, Object> unmarshallPrivate(String xml, Class<? extends Property<?>> property){
		try{
			// we standard have an elements and a <valueEntryElements> and <valueEntryRemovable>
			String first = xml.substring(xml.indexOf("<valueEntryElements>") + "<valueEntryElements>".length(), xml.indexOf("</valueEntryElements>"));
			String second = xml.substring(xml.indexOf("<valueEntryRemovable>") + "<valueEntryRemovable>".length(), xml.indexOf("</valueEntryRemovable>"));
			return new Pair<Object, Object>(property.newInstance().unmarshall(XMLHelper.unescape(first)), property.newInstance().unmarshall(XMLHelper.unescape(second)));
		} catch (Exception e){
			e.printStackTrace();
		}
		return null;
	}
	
	public boolean isValidConfiguration(Object elements, ConfigurablePropertyConfiguration<List<K>> config, Map<Class<? extends ConfigurableProperty<?>>, ?> dependentProperties){
		return HelperFunctions.disjoint(config.getRemoved(), config.getNonRemoved());
	}

	@SuppressWarnings("unchecked")
	public Object instantiate(Object elements, ConfigurablePropertyConfiguration<List<K>> config) {
		Pair<List<K>, List<K>> elems = (Pair<List<K>, List<K>>) elements;
		List<K> values = new ArrayList<K>(elems.getFirst());
		List<K> removable = new ArrayList<K>(elems.getSecond());
		for(K k: config.getRemoved()){
			values.remove(k);
			removable.remove(k);
		}
		for(K k: config.getNonRemoved()){
			removable.remove(k);
		}
		return new Pair<List<K>, List<K>>(values, removable);
	}
	
	@SuppressWarnings("unchecked")
	public Object instantiateTotal(Object elements, ConfigurablePropertyConfiguration<List<K>> config) {
		Pair<List<K>, List<K>> elems = (Pair<List<K>, List<K>>) elements;
		if(isTotalConfiguration(elems, config)){
			Pair<List<K>, List<K>> inst = (Pair<List<K>, List<K>>) instantiate(elems, config);
			return inst.getFirst();
		}
		else{
			throw new NullPointerException("Not a total configuration");
		}
	}
		
	@SuppressWarnings("unchecked")
	public boolean isTotalConfiguration(Object elements, ConfigurablePropertyConfiguration<List<K>> config){
		Pair<List<K>, List<K>> elems = (Pair<List<K>, List<K>>) elements;
		List<K> removable = elems.getSecond();
		for(K k: removable){
			if(!((!config.getRemoved().isEmpty() && config.getRemoved().contains(k)) || (!config.getNonRemoved().isEmpty() && config.getNonRemoved().contains(k)))){
				return false;
			}
		}
		return true;
	}
	
	@SuppressWarnings("unchecked")
	public void makeTotalConfiguration(Object elements, ConfigurablePropertyConfiguration<List<K>> config){
		Pair<List<K>, List<K>> elems = (Pair<List<K>, List<K>>) elements;
		for(K k: elems.getSecond()){
			if(!config.getRemoved().contains(k)){
				config.getNonRemoved().add(k);
			}
		}
	}
	
	public Object makeNonConfigurable(Object elements) {
		@SuppressWarnings("unchecked")
		Pair<List<K>, List<K>> values = (Pair<List<K>, List<K>>) elements;
		return values.getFirst();
	}

	@SuppressWarnings("unchecked")
	public Object makeConfigurable(Object element) {
		Pair<List<K>, List<K>> values = new Pair<List<K>, List<K>>(new ArrayList<K>(), new ArrayList<K>());
		values.getFirst().addAll((List<K>) element);
		return values;
	}
	
}
