package org.processmining.plugins.properties.processmodel.configurable.abstractproperty;

import java.util.ArrayList;
import java.util.List;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.processmodel.impl.Distribution;

/**
 * 
 * @author DSchunse
 *
 */
@SuppressWarnings("serial")
public abstract class ConfigurablePropertyListDistribution extends ConfigurablePropertyList<Distribution>{

	@SuppressWarnings("unchecked")
	public Pair<List<Distribution>, List<Distribution>> clone(Object element) {
		Pair<List<Distribution>, List<Distribution>> elements = (Pair<List<Distribution>, List<Distribution>>) element;
		List<Distribution> first = new ArrayList<Distribution>();
		List<Distribution> second = new ArrayList<Distribution>();
		for(Distribution d: elements.getFirst()){
			first.add(new Distribution(d));
		}
		for(Distribution d: elements.getSecond()){
			second.add(new Distribution(d));
		}		
		return new Pair<List<Distribution>, List<Distribution>>(first, second);
	}

	public Pair<List<Distribution>, List<Distribution>> getDefaultValue() {
		return new Pair<List<Distribution>, List<Distribution>>(new ArrayList<Distribution>(), new ArrayList<Distribution>());
	}

	public Object unmarshall(String xml) {
		try{
			// first remove the newlines, these are annoying
			String xmlIntern = xml.replaceAll("\n", "");
			// first remove the <valueEntryElements> and <valueEntryRemovable> elements
			String s = xmlIntern.substring(xmlIntern.indexOf("<valueEntryElements>") + "<valueEntryElements>".length(), xmlIntern.indexOf("</valueEntryElements>"));
			String[] distributions = s.split("<valueEntry");
			// now parse the whole lot
			ArrayList<Distribution> first = new ArrayList<Distribution>();
			for(String part: distributions){
				if(!part.isEmpty()){
					first.add((Distribution) Distribution.class.newInstance().unmarshall("<valueEntry" + part));
				}
			}
			s = xmlIntern.substring(xmlIntern.indexOf("<valueEntryRemovable>") + "<valueEntryRemovable>".length(), xmlIntern.indexOf("</valueEntryRemovable>"));
			distributions = s.split("<valueEntry");
			ArrayList<Distribution> second = new ArrayList<Distribution>();
			for(String part: distributions){
				if(!part.isEmpty()){
					second.add((Distribution) Distribution.class.newInstance().unmarshall("<valueEntry" + part));
				}
			}
			return new Pair<ArrayList<Distribution>, ArrayList<Distribution>>(first, second);
		} catch (Exception e){
			e.printStackTrace();
		}
		return null;
	}
	
	@SuppressWarnings("unchecked")
	public String marshall(Object values){
		Pair<List<Distribution>, List<Distribution>> elements = (Pair<List<Distribution>, List<Distribution>>) values;
		String ret = "<valueEntryElements>\n";
		for(Distribution d: elements.getFirst()){
			ret += d.marshall(d);
		}
		ret += "</valueEntryElements>\n";
		ret += "<valueEntryRemovable>\n";
		for(Distribution d: elements.getSecond()){
			ret += d.marshall(d);
		}
		ret += "</valueEntryRemovable>\n";
		return ret;
	}
}
