package org.processmining.plugins.properties.processmodel.configurable.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.annotations.PropertableElementProperty;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.plugins.properties.processmodel.configurable.ConfigurablePropertyConfiguration;
import org.processmining.plugins.properties.processmodel.configurable.abstractproperty.ConfigurablePropertyListList;
import org.processmining.plugins.properties.processmodel.impl.DataValueMatrix;

/**
 * 
 * @author DSchunse
 *
 */
@PropertableElementProperty
public class ConfigurableDataValueMatrix extends ConfigurablePropertyListList{

	private static final long serialVersionUID = 6449985278270344578L;

	public Long getID() {
		return serialVersionUID;
	}

	public String getName() {
		return "Configurable Data Value Matrix";
	}

	@SuppressWarnings("unchecked")
	public Pair<List<List<Object>>, List<List<Object>>> clone(Object element) {
		List<List<Object>> first = clone(((Pair<List<List<Object>>, List<List<Object>>>)element).getFirst());
		List<List<Object>> second = clone(((Pair<List<List<Object>>, List<List<Object>>>)element).getSecond());
		return new Pair<List<List<Object>>, List<List<Object>>>(first, second);
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public Object instantiateTotal(Object elements,	ConfigurablePropertyConfiguration<List<List<Object>>> config) {
		List<List<Object>> elems = (List<List<Object>>) super.instantiateTotal(elements, config);
		return toDatastructure(elems);
	}
	
	public Class<? extends Property<?>> getNonConfigurableVariant() {
		return DataValueMatrix.class;
	}

	public Object unmarshall(String xml) {
		Pair<Object, Object> dataStructure = super.unmarshallPrivate(xml, DataValueMatrix.class);
		return new Pair<Object, Object>(fromDatastructure(dataStructure.getFirst()), fromDatastructure(dataStructure.getSecond()));
	}


	public String marshall(List<List<Object>> value) {
		try {
			return getNonConfigurableVariant().newInstance().marshall(toDatastructure(value));
		} catch (InstantiationException | IllegalAccessException e) {
			e.printStackTrace();
		}
		return "";
	}
	
	private List<List<Object>> clone(List<List<Object>> elements){
		List<List<Object>> ret = new ArrayList<List<Object>>();
		for(List<Object> elem: elements){
			List<Object> elemRet = new ArrayList<Object>();
			// we have a fixed order of elements
			elemRet.add(new String((String) elem.get(0)));
			elemRet.add(new String((String) elem.get(1)));
			elemRet.add(new Double((Double) elem.get(2)));
			ret.add(elemRet);
		}
		return ret;
	}
	
	@SuppressWarnings("unchecked")
	private Object toDatastructure(Object elements){
		List<List<Object>> elems = (List<List<Object>>) elements;
		Map<Pair<String, String>, Double> structure = new HashMap<Pair<String, String>, Double>();
		// now make the original data structure out of it again
		for(List<Object> objs: elems){
			String low = (String) objs.get(0);
			String high = (String) objs.get(1);
			Double prob = (Double) objs.get(2);
			structure.put(new Pair<String, String>(low, high), prob);
		}
		return structure;
	}
	
	@SuppressWarnings("unchecked")
	private Object fromDatastructure(Object elements){
		// we have a hashmap<Pair<String, String>, Double> to lets make arrays out of it
		Map<Pair<String, String>, Double> elems = (Map<Pair<String, String>, Double>) elements;
		List<List<Object>> ret = new ArrayList<List<Object>>();
		for(Pair<String, String> key: elems.keySet()){
			List<Object> obs = new ArrayList<Object>();
			obs.add(key.getFirst());
			obs.add(key.getSecond());
			obs.add(elems.get(key));
			ret.add(obs);
		}
		return ret;
	}
	
	public Object makeNonConfigurable(Object elements) {
		@SuppressWarnings("unchecked")
		Pair<Object, Object> values = (Pair<Object, Object>) elements;
		return toDatastructure(values.getFirst());
	}

	public Object makeConfigurable(Object element) {
		Pair<Object, Object> values = new Pair<Object, Object>(fromDatastructure(element), fromDatastructure(element));
		return values;
	}

}
