package org.processmining.plugins.properties.processmodel.configurable.impl;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.annotations.PropertableElementProperty;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.plugins.properties.processmodel.configurable.ConfigurableProperty;
import org.processmining.plugins.properties.processmodel.configurable.ConfigurablePropertyConfiguration;
import org.processmining.plugins.properties.processmodel.configurable.abstractproperty.ConfigurablePropertyString;
import org.processmining.plugins.properties.processmodel.impl.DataValues;
import org.processmining.plugins.properties.processmodel.impl.InitialDataValue;

/**
 * 
 * @author DSchunse
 *
 */
@PropertableElementProperty
public class ConfigurableInitialDataValue extends ConfigurablePropertyString {

	private static final long serialVersionUID = 7583363657336823638L;

	public Long getID() {
		return serialVersionUID;
	}

	public String getName() {
		return "Configurable Initial Data Value";
	}
	
	public Collection<Class<? extends Property<?>>> getDependencies(){
		Collection<Class<? extends Property<?>>> dep = new HashSet<Class<? extends Property<?>>>();
		dep.add(ConfigurableDataValues.class);
		return dep;
	}
	
	public Class<? extends Property<?>> getNonConfigurableVariant() {
		return InitialDataValue.class;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public boolean isValidConfiguration(Object elements, ConfigurablePropertyConfiguration<List<String>> config, Map<Class<? extends ConfigurableProperty<?>>, ?> dependentProperties){
		Pair<List<String>, List<String>> elems = (Pair<List<String>, List<String>>) elements;
		if(!super.isValidConfiguration(elements, config, dependentProperties)){
			return false;
		}
		// now check that all the values are in the list of values (check configurable and non-configurable
		List<String> confDataVals = (List<String>) dependentProperties.get(ConfigurableDataValues.class);
		if(confDataVals == null){
			confDataVals = (List<String>) dependentProperties.get(DataValues.class);
		}
		for(String s: elems.getFirst()){
			// so we can still select an initial value which is no longer in the list of possible values 
			if(!config.getRemoved().contains(s) && !confDataVals.contains(s)){
				return false;
			}
		}
		return true;
	}

}
