package org.processmining.plugins.properties.processmodel.impl;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import javax.swing.ImageIcon;

import org.processmining.framework.plugin.annotations.KeepInProMCache;
import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.annotations.DataPerspective;
import org.processmining.plugins.properties.annotations.PropertableElementProperty;
import org.processmining.plugins.properties.enumerations.Constructs;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.plugins.properties.processmodel.abstractproperty.HighLevelProperty;
import org.processmining.plugins.properties.processmodel.gui.GUIProperty;
import org.processmining.plugins.properties.processmodel.gui.impl.GUIPropertyDataValueMatrix;

/**
 * 
 * @author DSchunse
 *
 */
@KeepInProMCache
@DataPerspective
@PropertableElementProperty
public class DataValueMatrix extends HighLevelProperty<HashMap<Pair<String, String>, Double>>{
	
	private static final long serialVersionUID = -118771733070284461L;

	public Long getID() {
		return serialVersionUID;
	}

	public String getName() {
		return "DataValueMatrix";
	}
	
	public Collection<Class<? extends Property<?>>> getDependencies(){
		Collection<Class<? extends Property<?>>> dep = new HashSet<Class<? extends Property<?>>>();
		dep.add(DataValues.class);
		return dep;
	}
	
	@SuppressWarnings("unchecked")
	public HashMap<Pair<String, String>, Double> clone(Object element){
		HashMap<Pair<String, String>, Double> elem = (HashMap<Pair<String, String>, Double>) element;
		HashMap<Pair<String, String>, Double> ret = new HashMap<Pair<String, String>, Double>();
		for(Pair<String, String> ss: elem.keySet()){
			ret.put(new Pair<String, String>(new String(ss.getFirst()), ss.getSecond()), new Double(elem.get(ss)));
		}
		return ret;
	}

	public int compare(Object o1, Object o2) {
		return 0;
	}

	public HashMap<Pair<String, String>, Double> getDefaultValue() {
		return new HashMap<Pair<String,String>, Double>();
	}

	public double getValueDouble(Object value) {
		return 0;
	}

	public ImageIcon getIcon() {
		return null;
	}

	public boolean hasOrdering() {
		return false;
	}

	public boolean higherBetter() {
		return false;
	}

	@SuppressWarnings("unchecked")
	public String marshall(Object values){
		HashMap<Pair<String, String>, Double> value = (HashMap<Pair<String, String>, Double>) values;
		String ret = "";
		for(Pair<String, String> key: value.keySet()){
			ret += "<valueEntry first=\"" + key.getFirst() + "\" second=\"" + key.getSecond() + "\" value=\"" + value.get(key) + "\" />\n"; 
		}
		return ret;
	}

	public Object unmarshall(String xml){
		HashMap<Pair<String, String>, Double> values = new HashMap<Pair<String, String>, Double>();
		// we have a set of <value things
		String[] valueString = xml.split("<valueEntry");
		for(String s: valueString){
			if(!s.isEmpty()){
				// we should do this nicer and more generic...?
				s = s.split("first")[1];
				String first = s.substring(s.indexOf("\"") + 1, s.indexOf("\"", s.indexOf("\"") + 1));
				s = s.split("second")[1];
				String second = s.substring(s.indexOf("\"") + 1, s.indexOf("\"", s.indexOf("\"") + 1));
				s = s.split("value")[1];
				Double value = Double.valueOf(s.substring(s.indexOf("\"") + 1, s.indexOf("\"", s.indexOf("\"") + 1)));
				values.put(new Pair<String, String>(first, second), value);
			}
		}
		return values;
	}

	public GUIProperty<HashMap<Pair<String, String>, Double>> getGUIProperty() throws InstantiationException, IllegalAccessException {
		return GUIPropertyDataValueMatrix.class.newInstance();
	}

	public Set<Constructs> getMeaningfulTypeConstructs() {
		Set<Constructs> constructs = new HashSet<Constructs>();
		constructs.add(Constructs.Variable);
		return constructs;
	}

}
