package org.processmining.plugins.properties.processmodel.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.processmining.framework.plugin.annotations.KeepInProMCache;
import org.processmining.plugins.properties.annotations.ExperimentPerspective;
import org.processmining.plugins.properties.annotations.PropertableElementProperty;
import org.processmining.plugins.properties.enumerations.Constructs;
import org.processmining.plugins.properties.enumerations.LikeFactor;
import org.processmining.plugins.properties.enumerations.QueueType;
import org.processmining.plugins.properties.processmodel.abstractproperty.HighLevelProperty;
import org.processmining.plugins.properties.processmodel.gui.GUIProperty;
import org.processmining.plugins.properties.processmodel.gui.impl.GUIPropertySimulationProperty;

/**
 * 
 * @author DSchunse
 *
 */
@KeepInProMCache
@ExperimentPerspective
@PropertableElementProperty
public class SimulationProperty extends HighLevelProperty<SimulationProperty> {

	private static final long serialVersionUID = 3142705774461179331L;

	QueueType qType = QueueType.FiFo;
	boolean Push = true;
	boolean Pull = !Push;
	List<LikeFactor> activity = new ArrayList<LikeFactor>();
	List<LikeFactor> originator = new ArrayList<LikeFactor>();
	int warmUpPeriod = 0;
	int nrSubRuns = 10;
	int lengthOfSubRuns = 100;
	
	public SimulationProperty(){}
	
	public SimulationProperty(SimulationProperty simProp){
		this.setqType(simProp.getqType());
		this.setPush(simProp.isPush());
		this.setPull(simProp.isPull());
		List<LikeFactor> lfs = new ArrayList<LikeFactor>();
		for(LikeFactor lf: simProp.getActivity()){
			lfs.add(lf);
		}
		this.setActivity(lfs);
		lfs = new ArrayList<LikeFactor>();
		for(LikeFactor lf: simProp.getOriginator()){
			lfs.add(lf);
		}
		this.setOriginator(lfs);
		this.setWarmUpPeriod(simProp.getWarmUpPeriod());
		this.setNrSubRuns(simProp.getNrSubRuns());
		this.setLengthOfSubRuns(simProp.getLengthOfSubRuns());
	}
	
	public QueueType getqType() {
		return qType;
	}

	public void setqType(QueueType qType) {
		this.qType = qType;
	}

	public boolean isPush() {
		return Push;
	}

	public void setPush(boolean push) {
		Push = push;
		Pull = !Push;
	}

	public boolean isPull() {
		return Pull;
	}

	public void setPull(boolean pull) {
		Pull = pull;
		Push = !Pull;
	}

	public List<LikeFactor> getActivity() {
		return activity;
	}

	public void setActivity(List<LikeFactor> activity) {
		this.activity = activity;
	}

	public List<LikeFactor> getOriginator() {
		return originator;
	}

	public void setOriginator(List<LikeFactor> originator) {
		this.originator = originator;
	}

	public int getWarmUpPeriod() {
		return warmUpPeriod;
	}

	public void setWarmUpPeriod(int warmUpPeriod) {
		this.warmUpPeriod = warmUpPeriod;
	}

	public int getNrSubRuns() {
		return nrSubRuns;
	}

	public void setNrSubRuns(int nrSubRuns) {
		this.nrSubRuns = nrSubRuns;
	}

	public int getLengthOfSubRuns() {
		return lengthOfSubRuns;
	}

	public void setLengthOfSubRuns(int lengthOfSubRuns) {
		this.lengthOfSubRuns = lengthOfSubRuns;
	}

	public Long getID() {
		return serialVersionUID;
	}

	public String getName() {
		return "SimulationProperty";
	}

	public int compare(Object o1, Object o2) {
		return 0;
	}

	public SimulationProperty getDefaultValue() {
		SimulationProperty simprop = new SimulationProperty();
		return simprop;
	}
	
	public SimulationProperty clone(Object element){
		return new SimulationProperty((SimulationProperty) element);
	}

	public String marshall(Object value) {
		SimulationProperty simprop = (SimulationProperty)value;
		String ret = "";
		ret += "<valueEntry queueType=\"" + simprop.qType + "\"/>\n";
		ret += "<valueEntry pull=\"" + simprop.Pull + "\"/>\n";
		ret += "<valueEntry push=\"" + simprop.Push + "\"/>\n";
		ret += "<valueEntry warmup=\"" + simprop.warmUpPeriod + "\"/>\n";
		ret += "<valueEntry nrRep=\"" + simprop.nrSubRuns + "\"/>\n";
		ret += "<valueEntry repLength=\"" + simprop.lengthOfSubRuns + "\"/>\n";
		// TODO: finish me
		return ret;
	}

	public Object unmarshall(String xml) {
		SimulationProperty simprop = new SimulationProperty();
		simprop.qType = QueueType.valueOf(xml.split("<valueEntry queueType=\"")[1].split("\"")[0]);
		simprop.Pull = Boolean.valueOf(xml.split("<valueEntry pull=\"")[1].split("\"")[0]);
		simprop.Push = Boolean.valueOf(xml.split("<valueEntry push=\"")[1].split("\"")[0]);
		simprop.warmUpPeriod = Integer.valueOf(xml.split("<valueEntry warmup=\"")[1].split("\"")[0]);
		simprop.nrSubRuns = Integer.valueOf(xml.split("<valueEntry nrRep=\"")[1].split("\"")[0]);
		simprop.lengthOfSubRuns = Integer.valueOf(xml.split("<valueEntry repLength=\"")[1].split("\"")[0]);
		return simprop;
	}

	public GUIProperty<SimulationProperty> getGUIProperty() throws InstantiationException, IllegalAccessException {
		return GUIPropertySimulationProperty.class.newInstance();
	}

	public Set<Constructs> getMeaningfulTypeConstructs() {
		Set<Constructs> constructs = new HashSet<Constructs>();
		constructs.add(Constructs.Model);
		return constructs;
	}

}
